@@ -6,6 +6,7 @@ import time
 from enum import Enum
 from itertools import cycle
 from pathlib import Path
+from textwrap import dedent
 from typing import Any, Dict, List, Optional, Union
 
 import fastar
@@ -32,6 +33,19 @@ from fastapi_cloud_cli.utils.pydantic_compat import (
 logger = logging.getLogger(__name__)
 
 
+def _cancel_upload(deployment_id: str) -> None:
+    logger.debug("Cancelling upload for deployment: %s", deployment_id)
+
+    try:
+        with APIClient() as client:
+            response = client.post(f"/deployments/{deployment_id}/upload-cancelled")
+            response.raise_for_status()
+
+            logger.debug("Upload cancellation notification sent successfully")
+    except Exception as e:
+        logger.debug("Failed to notify server about upload cancellation: %s", e)
+
+
 def _get_app_name(path: Path) -> str:
     # TODO: use pyproject.toml to get the app name
     return path.name
@@ -388,12 +402,15 @@ def _wait_for_deployment(
 
                     last_message_changed_at = time.monotonic()
 
-        except (BuildLogError, TooManyRetriesError) as e:
-            logger.error("Build log streaming failed: %s", e)
-            toolkit.print_line()
-            toolkit.print(
-                f"⚠️  Unable to stream build logs. Check the dashboard for status: [link={deployment.dashboard_url}]{deployment.dashboard_url}[/link]"
+        except (BuildLogError, TooManyRetriesError, TimeoutError) as e:
+            progress.set_error(
+                dedent(f"""
+                [error]Build log streaming failed: {e}[/]
+
+                Unable to stream build logs. Check the dashboard for status: [link={deployment.dashboard_url}]{deployment.dashboard_url}[/link]
+                """).strip()
             )
+
             raise typer.Exit(1) from e
 
 
@@ -598,15 +615,19 @@ def deploy(
                 logger.debug("Creating deployment for app: %s", app.id)
                 deployment = _create_deployment(app.id)
 
-                progress.log(
-                    f"Deployment created successfully! Deployment slug: {deployment.slug}"
-                )
+                try:
+                    progress.log(
+                        f"Deployment created successfully! Deployment slug: {deployment.slug}"
+                    )
 
-                progress.log("Uploading deployment...")
+                    progress.log("Uploading deployment...")
 
-                _upload_deployment(deployment.id, archive_path)
+                    _upload_deployment(deployment.id, archive_path)
 
-                progress.log("Deployment uploaded successfully!")
+                    progress.log("Deployment uploaded successfully!")
+                except KeyboardInterrupt:
+                    _cancel_upload(deployment.id)
+                    raise
 
         toolkit.print_line()
 
